#include <string.h>
#include <glpk.h>
#include <math.h>
#include <lauxlib.h>

typedef void (* setfn_t)(glp_prob *, int, int, double, double);

static void setbounds(lua_State *L, glp_prob *lp, int i, setfn_t setfn) {
  switch (lua_type(L, -1)) {
    case LUA_TNUMBER: {
      double k = luaL_checknumber(L, -1);
      glp_set_row_bnds(lp, i, GLP_FX, k, 0);
    }
    break;
    case LUA_TTABLE: {
      lua_getfield(L, -1, "type");
      const char *stype = luaL_checkstring(L, -1);
      lua_pop(L, 1);

      lua_rawgeti(L, -1, 1);
      double b1 = lua_tonumber(L, -1);
      lua_pop(L, 1);
      lua_rawgeti(L, -1, 2);
      double b2 = lua_tonumber(L, -1);
      lua_pop(L, 1);

      if (!strcmp(stype, "lo"))
        setfn(lp, i, GLP_LO, b1, 0);
      else if (!strcmp(stype, "up"))
        setfn(lp, i, GLP_UP, 0, b1);
      else if (!strcmp(stype, "db"))
        setfn(lp, i, GLP_DB, b1, b2);
      else if (!strcmp(stype, "fx"))
        setfn(lp, i, GLP_FX, b1, 0);
      else
        setfn(lp, i, GLP_FR, 0, 0);

      break;
    }
    default:
      luaL_error(L, "invalid type");
  }
}

static int solve(lua_State *L) {
  luaL_checktype(L, 1, LUA_TTABLE);
  luaL_checktype(L, 2, LUA_TTABLE);
  luaL_checktype(L, 3, LUA_TTABLE);
  luaL_checktype(L, 4, LUA_TTABLE);
  const char *dir = luaL_checkstring(L, 5);

  glp_prob *lp = glp_create_prob();
  if (!strcmp(dir, "max"))
    glp_set_obj_dir(lp, GLP_MAX);
  else
    glp_set_obj_dir(lp, GLP_MIN);

  lua_len(L, 2);
  int n = lua_tointeger(L, -1);
  lua_pop(L, 1);
  glp_add_cols(lp, n);

  for (int i = 1; i <= n; i++) {
    lua_rawgeti(L, 2, i);
    double k = luaL_checknumber(L, -1);
    lua_pop(L, 1);
    glp_set_obj_coef(lp, i, k);

    lua_rawgeti(L, 3, i);
    setbounds(L, lp, i, glp_set_col_bnds);
    lua_pop(L, 1);
  }

  lua_len(L, 4);
  int m = lua_tointeger(L, -1);
  lua_pop(L, 1);
  glp_add_rows(lp, m);

  for (int i = 1; i <= m; i++) {
    lua_rawgeti(L, 4, i);
    setbounds(L, lp, i, glp_set_row_bnds);
    lua_pop(L, 1);
  }

  int ia[1 + n * m];
  int ja[1 + n * m];
  double ka[1 + n * m];

  for (int i = 0; i < m; i++) {
    lua_rawgeti(L, 1, i + 1);
    for (int j = 0; j < n; j++) {
      ia[1 + i * n + j] = i + 1;
      ja[1 + i * n + j] = j + 1;
      lua_rawgeti(L, -1, j + 1);
      double k = luaL_checknumber(L, -1);
      lua_pop(L, 1);
      ka[1 + i * n + j] = k;
    }
    lua_pop(L, 1);
  }

  glp_load_matrix(lp, n * m, ia, ja, ka);

  glp_smcp param;
  glp_init_smcp(&param);
  param.msg_lev = GLP_MSG_OFF;
  glp_simplex(lp, &param);

  lua_pushnumber(L, glp_get_obj_val(lp));
  lua_createtable(L, n, 0);
  for (int i = 1; i <= n; i++) {
    lua_pushnumber(L, glp_get_col_prim(lp, i));
    lua_rawseti(L, -2, i);
  }

  glp_delete_prob(lp);

  return 2;
}

static const struct luaL_Reg libf[] = {
  {"solve", solve},
  {NULL, NULL},
};

int luaopen_glpk(lua_State *L) {
  luaL_newlib(L, libf);
  return 1;
}
