#!/usr/bin/env lua

local lg = require 'lg'
local util = require 'util'
local conf = require 'conf'
local plot = require 'plot'

math.randomseed(os.time())

local basedir = util.dir()

if conf.workdir then
    util.rm(conf.workdir, 'r')
    util.mkdir(conf.workdir)
    util.chdir(conf.workdir)
end

local model, rules = lg.new(conf.model)
local format = string.format

do
    local patchbox = {
        update = function (i, j, k, x)
            for _, e in ipairs(rules[i][j]) do
                if e.state == k then
                    e.chance = x
                end
            end
        end
    }
    assert(loadfile(basedir .. '/patch.lua', 't', patchbox))()
end

do
    local f = io.open('table.txt', 'w')
    for i, t in ipairs(rules) do
        if not conf.model.areas[i].ignore  then
            for j = 0, #t do
                local r = t[j]
                if not conf.model.areas[i].ignoreBoring or #r > 1 then
                    f:write(format('%d, %3d: ', i, j))
                    for k, e in ipairs(r) do
                        f:write(format('(%3d, %4.2f) ', e.state, e.chance))
                    end
                    f:write(format('\n'))
                end
            end
        end
    end
end

if conf.actions.calculate then
    local collect = lg.collect.all(conf.model.areas, conf.parameters)

    for iter = 1, conf.iterations do
        if (iter - 1) % conf.step == 0 then
            print(format('%d', iter))
        end
        model:step()
        collect(model.array, iter)
    end
end

--[[
for pk, p in pairs(conf.parameters) do
    for ok, o in pairs(p.outputs) do
        local base = format('data/%s/%s/', pk, o)
        local dir = format('paste -s `find %s -type f` > %s/table.txt', base, base)
        os.execute(dir)
    end
end
--]]

if conf.actions.plot then
    local plot = plot.all(conf)
    plot()
end
