local modname = ...
local M = {}
package.loaded[modname] = M
setmetatable(M, {__index = _G})
_ENV = M

local simple = {}
setmetatable(simple, {__index = _G})
_ENV = simple
dofile('simple.lua')
_ENV = M

iterations = simple.iterations

parameters = {
    density = {
        intervals = {
            {1, simple.iterations, simple.step},
        },
        format = "text",
        outputs = {'all', 'rp', 'mv'},
        vertical_slice = simple.vertical_slice,
    },
}

workdir = 'results'

actions = {
    calculate = simple.calculate,
    plot = simple.plot,
}

local size = simple.size

local floor = math.floor

local half = {}
for i, s in ipairs(size) do
    half[i] = floor(s / 2)
end

local areas = {
    {
        geometry = {
            {
                {0,           0},
                {size[1] - 1, size[2] - 1},
            },
        },
    },
    {
        geometry = {
            {
                {half[1] - simple.source.width, half[2] - simple.source.width},
                {half[1] + simple.source.width, half[2] + simple.source.width},
            },
        },
    },
}

for _, a in pairs(areas) do
    a.mv = {}
    a.rp = {}
end

local mv = {
    {mass = 1, pulse = {-1,  0}},
    {mass = 1, pulse = { 0,  1}},
    {mass = 1, pulse = { 1,  0}},
    {mass = 1, pulse = { 0, -1}},
}

for i, a in pairs(areas) do
    local b = simple.areas[i]

    a.ignore = b.ignore
    a.ignore_boring = b.ignore_boring

    for j, p in ipairs(mv) do
        a.mv[j] = {}
        a.mv[j].appear = b.mv[j]
        for k, v in pairs(p) do
            a.mv[j][k] = v
        end
    end

    for j, p in ipairs(b.rp) do
        a.rp[j] = {}
        a.rp[j].appear = b.rp[j]
        a.rp[j].mass = 2 ^ j
    end
end

local hpprp = {
    size = size,
    areas = areas,
}

model = hpprp
