#!/usr/bin/env lua

local lg = require 'lg'
local util = require 'util'
local conf = require 'conf'
local plot = require 'plot'

math.randomseed(os.time())

local basedir = util.dir()

if conf.workdir and conf.actions.calculate then
    util.rm(conf.workdir, 'r')
    util.mkdir(conf.workdir)
end
util.chdir(conf.workdir)

local model, rules = lg.new(conf.model)
local format = string.format

local patchbox = {
    update = function (i, j, k, x)
        for _, e in ipairs(rules[i][j]) do
            if e.state == k then
                e.chance = x
            end
        end
    end
}
assert(loadfile(basedir .. '/patch.lua', 't', patchbox))()

local f = io.open('table.txt', 'w')
for i, t in ipairs(rules) do
    if not conf.model.areas[i].ignore  then
        for j = 0, #t do
            local r = t[j]
            if not conf.model.areas[i].ignore_boring or #r > 1 then
                f:write(format('%d, %3d: ', i, j))
                for k, e in ipairs(r) do
                    f:write(format('(%3d, %4.2f) ', e.state, e.chance))
                end
                f:write(format('\n'))
            end
        end
    end
end

if conf.actions.calculate then
    local collect = lg.collect.all(conf.model.areas, conf.parameters)

    io.stdout:setvbuf('no')
    for iter = 1, conf.iterations do
        io.write(format('\r%d', iter))
        io.stdout:flush()
        model:step()
        collect(model.array, iter)
    end
    io.write('\r')

    for pk, p in pairs(conf.parameters) do
        for ok, o in pairs(p.outputs) do
            local base = format('data/%s/%s/', pk, o)
            local dir = format('paste -s `find %s -type f` > %s/table.txt', base, base)
            os.execute(dir)
        end
    end
end

if conf.actions.plot then
    plot.all(conf)()
end
