local modname = ...
local M = {}
package.loaded[modname] = M
setmetatable(M, {__index = _G})
_ENV = M

local util = require 'util'
local format =
[[
set terminal png
set output 'images/%s.png'
set tics nomirror
set border 3
set grid
set xrange[0:%d]
set yrange[0:%d]
]]

local function maxmass(conf)
    local max = 0
    for i, area in ipairs(conf.model.areas) do
        local ps = util.iconcat(area.mv, area.rp)
        local mass = 0
        for i, p in ipairs(ps) do
            mass = mass + p.mass
        end
        max = math.max(max, mass)
    end
    return max + 1
end

function distribution(conf)
    local maxmass = maxmass(conf)
    return function(param, out)
        tmp = os.tmpname()
        io.output(tmp)
        local filename = string.format('%s/%s/distribution', param, out)
        local size = conf.parameters.density.vertical_slice
            and conf.model.size[1]
            or  conf.model.size[2]
        io.write(string.format(format, filename, size, maxmass))
        local h = false
        for file in io.popen('ls data/' .. param .. '/' .. out .. ' | grep .dat'):lines() do
            if h then
                io.write(', ')
            else
                io.write('plot')
            end
            io.write(string.format("'data/%s/%s/%s' with lines notitle", param, out, file))
            h = true
        end
        io.flush()
        os.execute(string.format('gnuplot %s', tmp))
        os.remove(tmp)
        io.input()
        io.output()
    end
end

--[[
function progress(conf)
    local maxmass = maxmass(conf)
    return function(param, out)
        tmp = os.tmpname()
        data = os.tmpname()
        io.output(tmp)
        local filename = string.format('%s/%s/progress', param, out)
        io.write(string.format(format, filename, conf.iterations, maxmass))
        io.write(string.format("plot '%s' with lines notitle", data))
        io.output(data)
        for file in io.popen('ls data/' .. param .. '/' .. out .. ' | grep .dat'):lines() do
            local f = io.open(string.format('data/%s/%s/%s', param, out, file), 'r')
            local max = 0
            while true do
                n = f:read('*number')
                if not n then
                    break
                end
                max = math.max(max, n)
            end
            f:close()
            io.write(string.format('%s %f\n', file, max))
        end
        io.flush()
        os.execute(string.format('gnuplot %s', tmp))
        os.remove(tmp)
        os.remove(data)
        io.input()
        io.output()
    end
end
--]]

--[[
function sum(conf)
    local maxmass = maxmass(conf)
    return function(param, out)
        tmp = os.tmpname()
        data = os.tmpname()
        io.output(tmp)
        local filename = string.format('%s/%s/sum', param, out)
        io.write(string.format(format, filename, conf.iterations, maxmass))
        io.write(string.format("plot '%s' with lines notitle", data))
        io.output(data)
        for file in io.popen('ls data/' .. param .. '/' .. out .. ' | grep .dat'):lines() do
            local f = io.open(string.format('data/%s/%s/%s', param, out, file), 'r')
            local sum = 0
            while true do
                n = f:read('*number')
                if not n then
                    break
                end
                sum = sum + n
            end
            f:close()
            io.write(string.format('%s %f\n', file, sum / conf.model.size[1]))
        end
        io.flush()
        os.execute(string.format('gnuplot %s', tmp))
        os.remove(tmp)
        os.remove(data)
        io.input()
        io.output()
    end
end
--]]

function all(conf)
    util.mkdir('images')
    local distribution = distribution(conf)
--    local progress = progress(conf)
--    local sum = sum(conf)
    return function ()
        for param, opt in pairs(conf.parameters) do
            util.mkdir('images/' .. param)
            for _, out in ipairs(opt.outputs) do
                util.mkdir('images/' .. param .. '/' .. out)
                distribution(param, out)
--                progress(param, out)
--                sum(param, out)
            end
        end
    end
end
