#!/usr/bin/env lua

local lg = require 'lg'
local util = require 'util'
local conf = require 'conf'
local plot = require 'plot'

math.randomseed(os.time())

if conf.workdir then
    util.rm(conf.workdir, 'r')
    util.mkdir(conf.workdir)
    util.chdir(conf.workdir)
end

local model, rules = lg.new(conf.model)
local format = string.format

local function update(i, j, k, x)
    for _, e in ipairs(rules[i][j]) do
        if e.state == k then
            e.chance = x
        end
    end
end

local step = 0.5

io.stdout:setvbuf('line')
for c1 = 0.0, 1.0, step do
    for c2 = 0.0, 1.0, step do
        local dir = format('[%.1f  %.1f  %.1f  %.1f]', c1, 1.0 - c1, c2, 1.0 - c2)
        for i = 1, 2 do
            update(i, 5, 5, c1)
            update(i, 5, 10, 1.0 - c1)
            update(i, 10, 5, c2)
            update(i, 10, 10, 1.0 - c2)
        end
        print(dir)
        util.mkdir(dir)
        util.chdir(dir)
--[[
        local f = io.open('table.txt', 'w')
        for i, t in ipairs(rules) do
            if not conf.model.areas[i].ignore  then
                for j = 0, #t do
                    local r = t[j]
                    if not conf.model.areas[i].ignore_boring or #r > 1 then
                        f:write(format('%d, %3d: ', i, j))
                        for k, e in ipairs(r) do
                            f:write(format('(%3d, %4.2f) ', e.state, e.chance))
                        end
                        f:write(format('\n'))
                    end
                end
            end
        end
--]]
        if conf.actions.calculate then
            local collect = lg.collect.all(conf.model.areas, conf.parameters)

            for iter = 1, conf.iterations do
                io.write(format('\r%d', iter))
                io.stdout:flush()
                model:step()
                collect(model.array, iter)
            end
            io.write('\r')

            for pk, p in pairs(conf.parameters) do
                for ok, o in pairs(p.outputs) do
                    local base = format('data/%s/%s/', pk, o)
                    local dir = format('paste -s `find %s -type f` > %s/table.txt', base, base)
                    os.execute(dir)
                end
            end
        end
        if conf.actions.plot then
            local plot = plot.all(conf)
            plot()
        end
        util.chdir('..')
    end
end
